/*********************************************************************
 *
 * Includes Section
 *
 *********************************************************************/
#include <stdio.h>
#include "config.h"
#include "boards.h"
#include "app_util_platform.h"
#include "m24512.h"
#include "nrf_drv_twi.h"
#include "app_error.h"
#include "nrf_delay.h"

/*********************************************************************
 *
 * Defines Section
 *
 *********************************************************************/

/*********************************************************************
 *
 * Private Variables
 *
 *********************************************************************/
uint8_t m_device = EEPROM_HW_ADDR;             ///< save the device address
uint8_t m_device_id_page = DEVICE_ID_ADDR;  ///< save the device identification page address

uint8_t m_address[2];                       ///< used to send 16 bits device address

/* TWI instance. */
static const nrf_drv_twi_t m_twi_eeprom = NRF_DRV_TWI_INSTANCE(1);

/* Indicates if EEPROM operation has ended. */
static volatile bool m_xfer_done = true;

/**
 * @brief TWI events handler.
 */
void twi_handler(nrf_drv_twi_evt_t const * p_event, void * p_context)
{
#ifdef DEBUG_TWI
    dbg_printf("***twi_handler %d\r", p_event->type);
#endif
    
    switch(p_event->type)
    {
        case NRF_DRV_TWI_RX_DONE:
					nrf_delay_ms(10);
						m_xfer_done = true;
#ifdef DEBUG_TWI
            dbg_printf("NRF_DRV_TWI_RX_DONE\r");
#endif
            break;
        case NRF_DRV_TWI_TX_DONE:
						m_xfer_done = true;
#ifdef DEBUG_TWI
            dbg_printf("NRF_DRV_TWI_TX_DONE\r");
#endif
            break;
				case NRF_DRV_TWI_ERROR:
#ifdef DEBUG_TWI
						dbg_printf("NRF_DRV_TWI_ERROR\r");
#endif
						break;
        default:
            break;        
    }   
}

/**
 * @brief TWI initialization.
 */
void twi_init (void)
{
    ret_code_t err_code;
    
    const nrf_drv_twi_config_t twi_eeprom_config = {
       .scl                = I2C_SCK_PIN,
       .sda                = I2C_SDA_PIN,
       .frequency          = TWI1_CONFIG_FREQUENCY,
       .interrupt_priority = APP_IRQ_PRIORITY_HIGH
    };
    
    err_code = nrf_drv_twi_init(&m_twi_eeprom, &twi_eeprom_config, (nrf_drv_twi_evt_handler_t)twi_handler);
    APP_ERROR_CHECK(err_code);
    
    nrf_drv_twi_enable(&m_twi_eeprom);
}



/**@brief Function reads (length) byte data from (address) of M24512 EEPROM into (p_data) pointed memory.
 *
 * @param[in]   address                the memory address in the M24512 EEPROM (0 ~ 65535).
 * @param[out]  p_data                 an address in which the read data should be saved.
 * @param[in]   length                 the length of the data to be read.
 * @return      NRF_ERROR_I2C_TRANSFER error occured during the read
 *              NRF_SUCCESS            read is successful
 */
uint32_t m24512_read(uint16_t address, uint8_t *p_data, uint16_t length)
{
  bool res;
  
  //1. write the address to the m24512
  m_address[0] = (address>>8)&0xFF;
  m_address[1] = address&0xFF;
	
	m_xfer_done = false;
  res = nrf_drv_twi_tx(&m_twi_eeprom, EEPROM_HW_ADDR, m_address, 2, false);
	twi_wait();
	
	if(res==NRF_SUCCESS)
  {
		dbg_printf("TWI read length: %d\r", length);
    //2. read length data from m24512
		m_xfer_done = false;
		res = nrf_drv_twi_rx(&m_twi_eeprom, EEPROM_HW_ADDR, p_data, length, false);
		twi_wait();
    if(res==NRF_SUCCESS)
		{
			return NRF_SUCCESS;
		}
		else
		{
			dbg_printf("Error reading I2C data!\r");
			return NRF_ERROR_INVALID_DATA;
		}
  }
	else
	{
		dbg_printf("Error transmitting I2C read address %d!\r", res);
		return NRF_DRV_TWI_ERROR;
	}
}

/**@brief Function writes (length) byte data from (p_data) pointed memory into (address) of M24512 EEPROM.
 *
 * @param[in]   address                the memory address in the M24512 EEPROM (0 ~ 65535).
 * @param[out]  p_data                 a pointer to the data to be written.
 * @param[in]   length                 the length of the data to be written.
 * @return      NRF_ERROR_I2C_TRANSFER error occured during the write
 *              NRF_SUCCESS            write is successful
 */
uint32_t m24512_write(uint16_t address, uint8_t *p_data, uint16_t length)
{
  bool res;
  
  //1. write the @address to the m24512
  m_address[0] = (address>>8)&0xFF;
  m_address[1] = address&0xFF;
	m_xfer_done = false;
	res = nrf_drv_twi_tx(&m_twi_eeprom, EEPROM_HW_ADDR, m_address, 2, false);
	twi_wait();
	
  if(res==NRF_SUCCESS)
  {
		dbg_printf("TWI length: %d\r", length);
		//2. write @length data from @p_data to m24512, but don't send the device address
		m_xfer_done = false;
		res = nrf_drv_twi_tx(&m_twi_eeprom, EEPROM_HW_ADDR, p_data, length, false);
		twi_wait();
		if(res==NRF_SUCCESS)
		{
			return NRF_SUCCESS;
		}
		else
		{
			dbg_printf("Error transmitting I2C data!\r");
			return NRF_ERROR_INVALID_DATA;
		}
  }
	else
	{
		dbg_printf("Error transmitting I2C address %d!\r", res);
		return NRF_DRV_TWI_ERROR;
	}
  
}

void twi_wait(void)
{
		do
		{
				__WFE();
		}while(m_xfer_done == false);
}



/** @} */
